#!/usr/bin/perl
#  Copyright 2001-2016 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('IEP Report' => 'IEP Report',
	   'Name' => 'Name',
	   'Title' => 'Title',
	   'View/Download' => 'View/Download',
	   'View Log File' => 'View Log File',
	   'Parent/Guardian' => 'Parent/Guardian',
	   'Date' => 'Date',
	   'Main' => 'Main',
	   'Responsibility' => 'Responsibility',
	   'Long Term Objective' => 'Long Term Objective',
	   'Strategies, Resources, Activities' => 'Strategies, Resources, Activities',
	   'Category Name Missing!' => 'Category Name Missing!',
	   'Student Objectives' => 'Student Objectives',
	   'Student Team' => 'Student Team',
	   'Req' => 'Req',
	   'Act' => 'Act',
	   'Dates' => 'Dates',
	   'Category' => 'Category',
	   'Comments' => 'Comments',
	   'Assessments' => 'Assessments',
	   'Name' => 'Name',
	   'Identification' => 'Identification',
	   'Gender' => 'Gender',
	   'Birthdate' => 'Birthdate',
	   'Grade' => 'Grade',
	   'Designation Date' => 'Designation Date',
	   'DDP Level' => 'DDP Level',
	   'Designation' => 'Designation',
	   'Medical(other)' => 'Medical(other)',
	   'School' => 'School',
	   'Parent/Guardian' => 'Parent/Guardian',
	   'Address' => 'Address',
	   'Home Phone' => 'Home Phone',
	   'Emergency Contact' => 'Emergency Contact',
	   'Medical' => 'Medical',
	   'Medication' => 'Medication',
	   'Equipment' => 'Equipment',
	   'Adaptations' => 'Adaptations',
	   'History' => 'History',
	   'Personal Program Plan' => 'Personal Program Plan',
	   'Individual Education Plan' => 'Individual Education Plan',
	   'Student Impacts' => 'Student Impacts',
	   'Report' => 'Report',
	   'Error' => 'Error',
	   'Provincial Number' => 'Provincial Number',
	   'Please Log In' => 'Please Log In',
	   'Objectives' => 'Objectives',
	   'We, the undersigned, approve of this educational plan for' =>
	     'We, the undersigned, approve of this educational plan for',
	   'Signatures' => 'Signatures',
	   'Current Date' => 'Current Date',
	   'Student' => 'Student',
	   'Students' => 'Students',
	   'Select' => 'Select',
	   'Continue' => 'Continue',
	   
	   );

my $self = 'rptiep_local.pl';

# Configuration
my $showImpacts = 0; # (1=Yes, 0=No)
my $plantype = $lex{'Individual Education Plan'};


use DBI;
use CGI;
use CGI::Session;

my $q = new CGI;
my %arr = $q->Vars;

# Read config variables
eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "../../lib/liblatex.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "$globdir/global.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


# Now set the date;
my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
 $yday, $iddst) = localtime(time);
$year = $year + 1900;
$mon++; $wday++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Get Session
my $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

if ( not $session->param('logged_in') ){
    $userid = $session->param('userid');
    print $q->header;
    print "$doctype\n<html><head>\n";
    print "<link rel=\"stylesheet\" href=\"$tchcss\" type=\"text/css\"></head><body>\n";
    print "[ <a href=\"tchhome\">$lex{Main}</a> ]\n";
    
    print "<form action=\"../tlogin.pl\" method=\"post\" style=\"display:inline;\">\n";
    print "<input type=\"hidden\" name=\"userid\" value=\"$userid\">\n";
    print "<input type=\"hidden\" name=\"script\" value=\"$self\">\n";
    print "<input type=\"submit\" value=\"". $lex{'Please Log In'}. "\">\n";
    print "</form></body></html>\n";
    exit;
}

my $userid = $session->param('userid');
my $schooldbase = $session->param('schooldbase');

# Reset Login time
my $duration = $session->param('duration');
$session->expire('logged_in', $duration );

print $session->header;


# Get User Name, Set Select for School
my $sth = $dbh->prepare("select lastname, firstname from staff where userid = ?");
$sth->execute( $userid );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
my ( $lastname, $firstname ) = $sth->fetchrow;


# Print Page Header
my $title = qq{$lex{Student} $lex{'IEP Report'}};
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$tchcss" type="text/css">\n};
print qq{</head><body>\n};
print qq{[ <a href="$iephome">$lex{Main}</a> ] $firstname $lastname\n};
print qq{<h1>$title</h1>\n};


if ( not $arr{page} ) {
    showStudents();
}
delete $arr{page};

# foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }



my @students;
foreach my $key ( sort keys %arr ) { 
    push @students, $arr{$key}; # push in student number 
}



# Report Starts here
my $shortname = "repsum$$";
my $fileName = "$shortname.tex";

open(TEX,">$fileName") || die "Can't open tex file\n";


# Preamble and Head of Document
print TEX <<"EOS1";  
\\documentclass[11pt,letterpaper,oneside]{article}
\\usepackage{array,graphicx,multicol,colortbl,inputenc}
\\inputencoding{latin1}
\\setlength{\\hoffset}{-0.3in}
\\setlength{\\voffset}{-0.5in}
\\setlength{\\headsep}{10pt}
\\setlength{\\headheight}{14pt}
\\setlength{\\topmargin}{0in}
\\setlength{\\textwidth}{7.2in}
\\setlength{\\textheight}{9.6in}
\\setlength{\\parindent}{0pt}
\\setlength{\\evensidemargin}{0in}
\\setlength{\\oddsidemargin}{0in}
\\setlength{\\extrarowheight}{4pt}
\\pagestyle{headings}

\\begin{document}
EOS1

    foreach my $studnum ( @students ) {

	# school data in @student, local data in @special;
	$sth = $dbh->prepare("select * from iep_student where studnum = ?");
	$sth->execute( $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my @special = $sth->fetchrow; 
	@special = latex_filter( @special );

	my $studname = "$special[2] $special[1]";


	my $sth1 = $dbh->prepare("select * from student where studnum = ?");
	$sth1->execute( $studnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my @student = $sth1->fetchrow;

	# print "Student:", @student, "<br>\n";
	
	@student = latex_filter( @student );


# Print the heading section 
print TEX <<"EOS2";
\\thispagestyle{empty}

\\begin{center}
\\vspace*{1.5in}\n
{\\sf\\huge {\\it $plantype}}% IEP or PPP
\\bigskip\n
{\\sf\\LARGE $divisionname}\n
\\medskip
{\\sf\\Large $schoolname}\n
\\bigskip\n
{\\sf\\Huge\\it $studname}\n
\\bigskip\n
{\\sf $lex{'Current Date'} {\\bf $currdate}}\n
\\end{center}
\\newpage


\\setcounter{page}{1}
\\markright{$schoolname -- $schoolyear \\hfill  $studname -- }

\\begin{center}

%\\includegraphics[width=4in]{divlogo.jpg}
%\\medskip
{\\sf\\huge {\\it $plantype}}% IEP or PPP

\\smallskip

{\\sf\\LARGE $divisionname}

%{\\sf $lex{'Current Date'} {\\bf $currdate}}
\\bigskip

{\\sf\\LARGE\\it $studname}

\\medskip

EOS2

print TEX "\\setlength{\\extrarowheight}{4pt}\n\\smallskip\n";
# End of heading section; Now put in ID, etc.

print TEX <<"ID";

\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline
\\multicolumn{2}{|c|}{\\rule[-2mm]{0mm}{8mm}\\sf\\LARGE $lex{Identification}} \\\\\\hline\\hline
\\hfill {\\bf $lex{Name}}& $studname \\\\\\hline
\\hfill {\\bf $lex{Gender}}& $student[7]\\\\\\hline
\\hfill {\\bf $lex{Birthdate}}& $student[8]\\\\\\hline
\\hfill {\\bf $lex{'Provincial Number'}}& $student[57]\\\\\\hline
\\hfill {\\bf $lex{Grade}}& $student[5]\\\\\\hline

%\\hfill {\\bf $lex{'Designation Date'}}& $special[6]\\\\\\hline
%\\hfill {\\bf $lex{'DDP Level'}}& $special[7]\\\\\\hline
\\hfill {\\bf $lex{Designation}}& $special[8]\\\\\\hline

\\hfill {\\bf $lex{'Medical(other)'}}& $student[18]\\\\\\hline
\\hfill {\\bf $lex{School}}& $schoolname  -- $g_phone{$schooldbase}\\\\\\hline

\\hfill {\\bf $lex{'Parent/Guardian'}}& $student[29]\\\\\\hline \n
ID

if ( $student[40] ) {  # Name 2 - Parent 2
    print TEX "\\hfill {\\bf $lex{'Parent/Guardian'}}& $student[40]\\\\\\hline \n";
}

print TEX  "\\hfill {\\bf $lex{Address}}& $student[34] $student[35] $student[36]\\\\\\hline
\\hfill {\\bf $lex{'Home Phone'}}& $student[30]\\\\\\hline
\\hfill {\\bf $lex{'Emergency Contact'}}& $student[49] $student[50] $student[51]\\\\\\hline
\\hfill {\\bf $lex{Medical}}& $special[12]\\\\\\hline
\\end{tabular}\n";

print TEX "\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline\n";
print TEX "\\hfill {\\bf $lex{Medication}}& $special[13]\\\\\\hline\n";

print TEX "\\end{tabular}\n\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline\n";
print TEX "\\hfill {\\bf $lex{Equipment}}& $special[14]\\\\\\hline\n";

print TEX "\\end{tabular}\n\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline\n";
print TEX "\\hfill {\\bf $lex{Adaptations}}& $special[15]\\\\\\hline\n";

print TEX "\\end{tabular}\n\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline\n";
print TEX "\\hfill {\\bf $lex{History}}& $special[16]\\\\\\hline\n";

print TEX "\\end{tabular}\n\n\\bigskip\n";


=head
# Let's Do Assessments!
$sth1 = $dbh->prepare("select * from assess  where studnum = ? order by resdate desc" );
$sth1->execute( $studnum );
if ($DBI::errstr) { print $DBI::errstr; die;}
$assessrows = $sth1->rows; 

# Now print out Assessment headings
print TEX "\\begin{tabular}{|p{1.1in}|p{1.8in}|p{3.8in}|}\\hline
\\multicolumn{3}{|c|}{\\sf\\LARGE ". $lex{Assessments}. " \\rule[-2mm]{0mm}{8mm} } \\\\ \\hline\\hline\n";


print TEX "{\\bf\\sf\\large ". $lex{Dates}. "}\n& {\\bf\\sf\\large ". $lex{Category};
print TEX "} & {\\bf\\sf\\large ". $lex{Comments}. "} \\\\ \\hline\n";

for ( 1..$assessrows){
    @assess = $sth1->fetchrow;
    @assess = latex_filter(@assess);
    print TEX $lex{Req}. ":~$assess[4] ". $lex{Act}. ":~$assess[3] & $assess[2] ";
    print TEX "& $assess[5] \\\\ \\hline \n";
}
print TEX "\\end{tabular}\\bigskip\n\n";
=cut

# TEAM - skip until ready.
=head
# Now lets do the team.
$sth1 = $dbh->prepare("select * from team where studnum = ? order by userid");
$sth1->execute( $studnum );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
$teamrows = $sth1->rows; 


print TEX "\\begin{tabular}{|p{3.5in}|p{3.3in}|}\\hline\n";
print TEX "\\multicolumn{2}{|c|}{\\sf\\LARGE ";
print TEX $lex{'Student Team'}. " \\rule[-2mm]{0mm}{8mm} } \\\\ \\hline\\hline\n";

print TEX "{\\bf\\sf\\large ". $lex{Name}. "}
& {\\bf\\sf\\large ". $lex{Title}. "} \\\\ \\hline\n";

for ( 1..$teamrows ) {
    my @team = $sth1->fetchrow;
    @team = latex_filter(@team);

    ($teammember,$dud) = split(/\(/,$team[2] );
    if ($team[6] eq 'Y' ){
	push @teamarray,$teammember;  # needed for signature page
    }
    print TEX "$teammember & $team[3] \\\\ \\hline \n";
}
print TEX "\\end{tabular}\\newpage\n\n";
=cut

print TEX "\\newpage\n\n";  # remove when you uncomment team section above.




# Start of IMPACTS
if ( $showImpacts ) {
    print TEX "\\begin{tabular}{|p{7in}|}\\hline\n";
    print TEX "\\sf\\LARGE \\hfil ". $lex{'Student Impacts'}. " \\rule[-2mm]{0mm}{8mm}";
    print TEX "\\hfil  \\\\ \\hline\\hline\n";

    $sth = $dbh->prepare("select id, area, description from impact where studnum = ? 
      order by area");
    $sth->execute( $studnum );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    
    while ( my ($id, $area, $description ) = $sth->fetchrow ) {
	( $description ) = latex_filter( $description );
	print TEX "{\\bf $area } $description \\\\\\hline\n";
    }
    print TEX "\\end{tabular}\n\\bigskip\n\n";
}
# End of IMPACTS



# Now find their objectives
$sth = $dbh->prepare("select distinct e.* from iep_evalmst as e, iep_objectives as o
  where e.subnum = o.catnum and e.studnum = ? 
  order by e.impactarea, o.category");
$sth->execute( $studnum );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }


print TEX "\\begin{tabular}{|p{7in}|}\\hline\n\\hfil {\\rule[-2mm]{0mm}{8mm} \\sf\\LARGE ";
print TEX $lex{'Student Objectives'}. "}\\hfil\\\\\\hline\n \\end{tabular}\n\n";


# Subjects Loop; once for each subject; they come out of evalmst.
my $currimpact = -1;
my $previmpact;

while ( my @evalmst = $sth->fetchrow ) {

    $previmpact = $currimpact;
    $currimpact = $evalmst[38]; # last field is impactarea.

#    if ( $currimpact ne $previmpact ) { # we have a new one; print title
#	print TEX "{\\LARGE $currimpact }\n\n";
#    }

    @evalmst = latex_filter( @evalmst );
    my $subnum = $evalmst[2];

    #lookup subject name, $subname
    my $sth1 = $dbh->prepare("select category from iep_objectives where catnum = ?");
    $sth1->execute( $subnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my $subname = $sth1->fetchrow;
    if ( not $subname ) {
	$subname = $lex{'Category Name Missing!'};
    }

    print TEX "\\smallskip\n\\begin{tabular}{|p{7in}|}\\hline\n\\rowcolor[gray]{0.85}";
    print TEX " \\hfil {\\Large\\sf $currimpact $subname}\\hfil";
    print TEX " \\rule[-2mm]{0mm}{8mm} \\\\ \\hline\n";

    #print TEX "{\\sf\\large\\bf ". $lex{'Area of Impact'}. "} $evalmst[38] \\\\ \\hline\n";

    print TEX "{\\sf\\large\\bf ". $lex{Responsibility}. "} $evalmst[36] \\\\ \\hline\n";
    print TEX "{\\sf\\large\\bf ". $lex{'Long Term Objective'}. "} $evalmst[37] \\\\ \\hline\n";
    print TEX "{\\sf\\large\\bf ". $lex{'Strategies, Resources, Activities'};
    print TEX "} $evalmst[35]\n\\bigskip \\\\ \\hline\n";

    # Now print the objectives; first check for presence of objectives...
    $objflag = 0;
    print TEX "{\\sf\\large ". $lex{Objectives}. "} \\\\\n";
    for ($j=3; $j<=34;$j++){
	if ($evalmst[$j]){
	    $objflag++;
	}
    } 
    if ($objflag){  # We have some objectives.
 	#print TEX "\n\\begin{itemize}\n";
	my $itemcount = 0;
	for ($j=3; $j<=34;$j++){
	    if ( $evalmst[$j] ) {
		print TEX "$evalmst[$j] \\\\  \n";
		#print TEX "\\item $evalmst[$j]\n";
		$itemcount++;
	    
		if (not ($itemcount % 3) and $itemcount < $objflag){
		#    print TEX "\n\\end{itemize}\n";		
		     print TEX "\\end{tabular}
                      \\begin{tabular}{|p{7in}|}\n";
		#    print TEX "\n\\begin{itemize}\n";
		}
	    }
	}
	#print TEX "\\end{itemize}\n";
    }
    print TEX "\\\\\\hline\\end{tabular}\\bigskip\n\n";

} # End of Objectives Loop


print TEX "\\end{center}\n\\newpage\n\n";


# Now the "team" records.
print TEX "{\\sf\\LARGE ". $lex{Signatures}. "}\n\\bigskip\n\n{\\Large ";
print TEX $lex{'We, the undersigned, approve of this educational plan for'};
print TEX " $studname}\n\n\\bigskip\n\n"; 

foreach my $teammember ( @teamarray ) {
    print TEX "\\underline{\\hspace{3in}}\\hfill \\underline{\\hspace{1.5in}}\n\n";
    print TEX "$teammember \\hfill ". $lex{Date}. " \\bigskip\n\n";
}

print TEX "\\underline{\\hspace{3in}} \\hfill \\underline{\\hspace{1.5in}} \\\\ \n";
print TEX $lex{'Parent/Guardian'}. " \\hfill ". $lex{Date}. " \\bigskip\n\n";
print TEX "\\underline{\\hspace{3in}} \\hfill \\underline{\\hspace{1.5in}} \\\\ \n";
print TEX $lex{'Parent/Guardian'}. " \\hfill ". $lex{Date}. "\n\n";
print TEX "\\newpage\n";

} # Loop through to next student;


# Print ending of report
print TEX "\\end{document}\n";
close TEX;


system("$pdflatex $fileName > pdflog$$.txt");

system("mv $shortname.pdf $tchdownloaddir");
system("mv pdflog$$.txt $tchdownloaddir");
system("rm -f $shortname.*");


print qq{<h3><a href=\"$tchwebdownloaddir/$shortname.pdf\">};
print qq{$lex{'View/Download'} $lex{'IEP Report'}</a></h3>\n};

print qq{<p>[ <a href=\"$iephome\">$lex{Main}</a> | \n};
print qq{<a href=\"$tchwebdownloaddir/pdflog$$.txt\">$lex{'View Log File'}</a> ]</p>\n};

print "</body></html>\n";


#---------------
sub showStudents {
#---------------

    my $sth = $dbh->prepare("select lastname, firstname, studnum, grade, school from iep_student
     order by lastname, firstname"); # $select is set by session code above to select only single school
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="1">\n};
    
    print qq{<table cellpadding="3" cellspacing="0" border="0" style="border:1px solid gray;padding:0.5em;">\n};
    print qq{<tr><th colspan="2">$lex{Select} $lex{Students}</th></tr>\n};
    print qq{<tr><td colspan="2"><input type="submit" value="$lex{Continue}"></td></tr>\n};

    while ( my ($lastname, $firstname, $studnum, $grade, $sch ) = $sth->fetchrow ){

	print qq{<tr><td class="la"><input type="checkbox" name="$lastname$firstname$studnum" value="$studnum"> };
	print qq{<b>$lastname</b>, $firstname</td><td class="la">$lex{Grade} $grade</td></tr>\n};
    }

    print qq{<tr><td colspan="2"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table></form></body></html>\n};

    exit;

} # End of showStudents
